/***************************************************************************
                          kplayer.cpp
                          -----------
    begin                : Sat Nov 16 10:12:50 EST 2002
    copyright            : (C) 2002-2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#define _ISOC99_SOURCE
#include <kapplication.h>
#include <kcmdlineargs.h>
#include <kconfig.h>
#include <klocale.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <kstdaction.h>
#include <kurl.h>
#include <math.h>
#include <qcursor.h>
#include <qlayout.h>
#include <qobjectlist.h>

#include <kdebug.h>

#include "kplayer.h"
#include "kplayerslideraction.h"

#define ID_STATUS_MSG   1
#define ID_STATE_MSG    2
#define ID_PROGRESS_MSG 3

int KPlayerApplication::newInstance (void)
{
  kdDebug() << "New instance\n";
  KPlayer* mainWindow = (KPlayer*) mainWidget();
  if ( ! mainWindow )
  {
    kdDebug() << "Creating main window\n";
    mainWindow = new KPlayer();
    setMainWidget (mainWindow);
  }
  mainWindow -> start();
  return 0;
}

KPlayer::KPlayer (QWidget *parent, const char *name) : KMainWindow (parent, name)
{
  m_resizing = true;
  m_updating = m_initial_fullscreen = false;
  m_status_label = m_state_label = m_progress_label = 0;
  m_menubar_normally_visible = m_statusbar_normally_visible = true;
  m_menubar_fullscreen_visible = m_statusbar_fullscreen_visible = false;
  Toolbar toolbar [KPLAYER_TOOLBARS] = {
    { "mainToolBar", KStdAction::stdName (KStdAction::ShowToolbar), true, false },
    { "progressToolBar", "settings_progress_toolbar", true, false },
    { "volumeToolBar", "settings_volume_toolbar", false, false },
    { "brightnessToolBar", "settings_brightness_toolbar", false, false },
    { "contrastToolBar", "settings_contrast_toolbar", false, false },
    { "hueToolBar", "settings_hue_toolbar", false, false },
    { "saturationToolBar", "settings_saturation_toolbar", false, false }
  };
  KPlayerWorkspace* workspace = new KPlayerWorkspace (this);
  connect (actionCollection(), SIGNAL (actionStatusText(const QString&)), this, SLOT (setStatusMessage(const QString&)));
  connect (actionCollection(), SIGNAL (clearStatusText()), this, SLOT (clearStatusMessage()));
  actionCollection() -> setHighlightingEnabled (true);
  connect (workspace -> kPlayerWidget(), SIGNAL (resized()), this, SLOT (onWidgetResized()));
  connect (workspace -> kPlayerProcess(), SIGNAL (stateChanged(KPlayerProcess::State)), this, SLOT (onPlayerStateChanged(KPlayerProcess::State)));
  connect (workspace -> kPlayerProcess(), SIGNAL (progressChanged(float)), this, SLOT (onPlayerProgressChanged(float)));
  connect (workspace -> kPlayerProcess(), SIGNAL (infoAvailable()), this, SLOT (onPlayerInfoAvailable()));
  setCentralWidget (workspace);
  setAutoSaveSettings ("MainWindow", false);
  resetAutoSaveSettings();  // saveWindowSize = false;
  initStatusBar();
  initActions();
  for ( int i = 0; i < KPLAYER_TOOLBARS; i ++ )
//{
    m_toolbar[i] = toolbar[i];
/*  connect (toolBar (m_toolbar[i].name), SIGNAL (orientationChanged(Orientation)), this, SLOT (onBarOrientationChanged(Orientation)));
    connect (toolBar (m_toolbar[i].name), SIGNAL (placeChanged(QDockWindow::Place)), this, SLOT (onBarPlaceChanged(QDockWindow::Place)));
    connect (toolBar (m_toolbar[i].name), SIGNAL (visibilityChanged(bool)), this, SLOT (onBarVisibilityChanged(bool)));
  }*/
  readOptions();
  for ( int i = 0; i < KPLAYER_TOOLBARS; i ++ )
    if ( toolBar (m_toolbar[i].name) -> orientation() != Qt::Horizontal )
    {
      // force orientationChanged event
      toolBar (m_toolbar[i].name) -> undock();
      toolBar (m_toolbar[i].name) -> dock();
      if ( ! toggleAction (m_toolbar[i].action) -> isChecked() )
        toolBar (m_toolbar[i].name) -> hide();
    }
  enablePlayerActions();
  enableVideoActions();
  setEraseColor (QColor (0, 0, 0));
  m_resizing = false;
  resize (450, 350);
//kPlayerWidget() -> setVideoSize (QSize (450, 0), true);
//zoom();
}

void KPlayer::initActions (void)
{
  KAction* action = KStdAction::open (this, SLOT (onFileOpen()), actionCollection());
  action -> setStatusText (i18n("Opens an existing file"));
  action = new KAction (i18n("Open &URL..."), "fileopenurl", CTRL + Key_U, this, SLOT (onFileOpenUrl()), actionCollection(), "file_open_url");
  action -> setStatusText (i18n("Opens a URL"));
  action = new KAction (i18n("&Load Subtitle..."), 0, CTRL + Key_L, this, SLOT (onFileOpenSubtitle()), actionCollection(), "file_open_subtitle");
  action -> setStatusText (i18n("Opens a subtitle file"));
  action = new KAction (i18n("Load Su&btitle URL..."), 0, 0, this, SLOT (onFileOpenSubtitleUrl()), actionCollection(), "file_open_subtitle_url");
  action -> setStatusText (i18n("Opens a subtitle URL"));
  action = new KToggleAction (i18n("U&nload Subtitles"), 0, 0, this, SLOT (onFileUnloadSubtitles()), actionCollection(), "file_unload_subtitles");
  action -> setStatusText (i18n("Unloads subtitles"));
  KRecentFilesAction* recent = KStdAction::openRecent (this, SLOT (onFileOpenRecent(const KURL&)), actionCollection());
  recent -> setStatusText (i18n("Opens a recently used file"));
  recent -> setMaxItems (10);
  recent -> setEnabled (false);
  action = KStdAction::quit (this, SLOT (onFileQuit()), actionCollection());
  action -> setStatusText (i18n("Quits KPlayer"));

  action = new KToggleAction (i18n("&Full Screen"), "window_fullscreen", CTRL + Key_F, this, SLOT (onViewFullScreen()), actionCollection(), "view_full_screen");
  action -> setStatusText (i18n("Switches to full screen mode"));
  action = KStdAction::zoomIn (this, SLOT (onViewZoomIn()), actionCollection());
  action -> setStatusText (i18n("Enlarges the video"));
  action = KStdAction::zoomOut (this, SLOT (onViewZoomOut()), actionCollection());
  action -> setStatusText (i18n("Shrinks the video"));
  KToggleAction* toggle = new KToggleAction (i18n("Zoom to 5&0%"), 0, 0, this, SLOT (onViewZoom12()), actionCollection(), "view_zoom_1_2");
  toggle -> setStatusText (i18n("Scales video to 50% of the original size"));
  toggle -> setExclusiveGroup ("zoom");
  toggle = new KToggleAction (i18n("Zoom to &100%"), 0, CTRL + Key_Backspace, this, SLOT (onViewZoom11()), actionCollection(), "view_zoom_1_1");
  toggle -> setStatusText (i18n("Scales video to its original size"));
  toggle -> setExclusiveGroup ("zoom");
  toggle = new KToggleAction (i18n("Zoo&m to 150%"), 0, 0, this, SLOT (onViewZoom32()), actionCollection(), "view_zoom_3_2");
  toggle -> setStatusText (i18n("Scales video to 150% of the original size"));
  toggle -> setExclusiveGroup ("zoom");
  toggle = new KToggleAction (i18n("Zoom to &200%"), 0, 0, this, SLOT (onViewZoom21()), actionCollection(), "view_zoom_2_1");
  toggle -> setStatusText (i18n("Scales video to 200% of the original size"));
  toggle -> setExclusiveGroup ("zoom");
  toggle = new KToggleAction (i18n("Zoom &to 250%"), 0, 0, this, SLOT (onViewZoom52()), actionCollection(), "view_zoom_5_2");
  toggle -> setStatusText (i18n("Scales video to 250% of the original size"));
  toggle -> setExclusiveGroup ("zoom");
  toggle = new KToggleAction (i18n("Zoom to &300%"), 0, 0, this, SLOT (onViewZoom31()), actionCollection(), "view_zoom_3_1");
  toggle -> setStatusText (i18n("Scales video to 300% of the original size"));
  toggle -> setExclusiveGroup ("zoom");
  toggle = new KToggleAction (i18n("Maintain Original &Aspect"), "viewmagfit", CTRL + Key_A, this, SLOT (onViewMaintainOriginalAspect()), actionCollection(), "view_original_aspect");
  toggle -> setStatusText (i18n("Maintains the original video aspect ratio"));
  toggle -> setExclusiveGroup ("aspect");
  toggle = new KToggleAction (i18n("Maintain &Current Aspect"), 0, 0, this, SLOT (onViewMaintainCurrentAspect()), actionCollection(), "view_current_aspect");
  toggle -> setStatusText (i18n("Maintains the current video aspect ratio"));
  toggle -> setExclusiveGroup ("aspect");
  toggle = new KToggleAction (i18n("Force &4 to 3 Aspect"), 0, 0, this, SLOT (onViewAspect43()), actionCollection(), "view_aspect_4_3");
  toggle -> setStatusText (i18n("Sets 4 to 3 video aspect ratio"));
  toggle -> setExclusiveGroup ("aspect");
  toggle = new KToggleAction (i18n("Force 1&6 to 9 Aspect"), 0, 0, this, SLOT (onViewAspect169()), actionCollection(), "view_aspect_16_9");
  toggle -> setStatusText (i18n("Sets 16 to 9 video aspect ratio"));
  toggle -> setExclusiveGroup ("aspect");

  action = new KAction (i18n("&Play"), "player_play", Key_Return, this, SLOT (onPlayerPlay()), actionCollection(), "player_play");
  action -> setStatusText (i18n("Plays the currently loaded file"));
  action = new KToggleAction (i18n("Paus&e"), "player_pause", Key_Space, this, SLOT (onPlayerPause()), actionCollection(), "player_pause");
  action -> setStatusText (i18n("Pauses the player"));
  action = new KAction (i18n("&Stop"), "player_stop", Key_Escape, this, SLOT (onPlayerStop()), actionCollection(), "player_stop");
  action -> setStatusText (i18n("Stops the player"));

  action = new KAction (i18n("&Forward"), 0, Key_Right, this, SLOT (onPlayerForward()), actionCollection(), "player_forward");
  action -> setStatusText (i18n("Moves forward by one percent of video length"));
  action = new KAction (i18n("F&ast Forward"), "player_fwd", CTRL + Key_Right, this, SLOT (onPlayerFastForward()), actionCollection(), "player_fast_forward");
  action -> setStatusText (i18n("Moves forward by ten percent of video length"));
  action = new KAction (i18n("&Backward"), 0, Key_Left, this, SLOT (onPlayerBackward()), actionCollection(), "player_backward");
  action -> setStatusText (i18n("Moves backward by one percent of video length"));
  action = new KAction (i18n("Fast Back&ward"), "player_rew", CTRL + Key_Left, this, SLOT (onPlayerFastBackward()), actionCollection(), "player_fast_backward");
  action -> setStatusText (i18n("Moves backward by ten percent of video length"));
  action = new KAction (i18n("Back &to start"), 0, CTRL + Key_Home, this, SLOT (onPlayerStart()), actionCollection(), "player_start");
  action -> setStatusText (i18n("Moves to the beginning"));

  action = new KAction (i18n("Volume &Up"), 0, Key_Up, this, SLOT (onPlayerVolumeUp()), actionCollection(), "player_volume_up");
  action -> setStatusText (i18n("Increases the volume"));
  action = new KAction (i18n("Volume &Down"), 0, Key_Down, this, SLOT (onPlayerVolumeDown()), actionCollection(), "player_volume_down");
  action -> setStatusText (i18n("Decreases the volume"));
  action = new KToggleAction (i18n("&Mute"), "mute", CTRL + Key_Backslash, this, SLOT (onPlayerMute()), actionCollection(), "player_mute");
  action -> setStatusText (i18n("Turns the sound on/off"));

  toggle = new KToggleAction (i18n("S&oft Frame Dropping"), 0, 0, this, SLOT (onPlayerSoftFrameDrop()), actionCollection(), "player_soft_frame_drop");
  toggle -> setStatusText (i18n("Turns soft frame dropping on/off"));
  toggle -> setExclusiveGroup ("framedrop");
  toggle = new KToggleAction (i18n("&Hard Frame Dropping"), 0, 0, this, SLOT (onPlayerHardFrameDrop()), actionCollection(), "player_hard_frame_drop");
  toggle -> setStatusText (i18n("Turns hard frame dropping on/off"));
  toggle -> setExclusiveGroup ("framedrop");

  action = new KAction (i18n("C&ontrast Down"), 0, CTRL + Key_1, this, SLOT (onVideoContrastDown()), actionCollection(), "video_contrast_down");
  action -> setStatusText (i18n("Decreases the video contrast"));
  action = new KAction (i18n("&Contrast Up"), 0, CTRL + Key_2, this, SLOT (onVideoContrastUp()), actionCollection(), "video_contrast_up");
  action -> setStatusText (i18n("Increases the video contrast"));
  action = new KAction (i18n("B&rightness Down"), 0, CTRL + Key_3, this, SLOT (onVideoBrightnessDown()), actionCollection(), "video_brightness_down");
  action -> setStatusText (i18n("Decreases the video brightness"));
  action = new KAction (i18n("&Brightness Up"), 0, CTRL + Key_4, this, SLOT (onVideoBrightnessUp()), actionCollection(), "video_brightness_up");
  action -> setStatusText (i18n("Increases the video brightness"));
  action = new KAction (i18n("H&ue Down"), 0, CTRL + Key_5, this, SLOT (onVideoHueDown()), actionCollection(), "video_hue_down");
  action -> setStatusText (i18n("Decreases the video hue"));
  action = new KAction (i18n("&Hue Up"), 0, CTRL + Key_6, this, SLOT (onVideoHueUp()), actionCollection(), "video_hue_up");
  action -> setStatusText (i18n("Increases the video hue"));
  action = new KAction (i18n("S&aturation Down"), 0, CTRL + Key_7, this, SLOT (onVideoSaturationDown()), actionCollection(), "video_saturation_down");
  action -> setStatusText (i18n("Decreases the video saturation"));
  action = new KAction (i18n("&Saturation Up"), 0, CTRL + Key_8, this, SLOT (onVideoSaturationUp()), actionCollection(), "video_saturation_up");
  action -> setStatusText (i18n("Increases the video saturation"));

  action = KStdAction::showMenubar (this, SLOT (onViewMenubar()), actionCollection());
  action -> setStatusText (i18n("Shows/hides the menu bar"));
  action = KStdAction::showToolbar (this, SLOT (onViewMainToolbar()), actionCollection());
  action -> setText (i18n("Show Main &Toolbar"));
  action -> setStatusText (i18n("Shows/hides the main toolbar"));
  action = new KToggleAction (i18n("Show &Progress Toolbar"), 0, 0, this, SLOT (onViewProgressToolbar()), actionCollection(), "settings_progress_toolbar");
  action -> setStatusText (i18n("Shows/hides the progress toolbar"));
  action = new KToggleAction (i18n("Show &Volume Toolbar"), "volume", 0, this, SLOT (onViewVolumeToolbar()), actionCollection(), "settings_volume_toolbar");
  action -> setStatusText (i18n("Shows/hides the volume toolbar"));
  action = new KToggleAction (i18n("Show &Brightness Toolbar"), "brightness", 0, this, SLOT (onViewBrightnessToolbar()), actionCollection(), "settings_brightness_toolbar");
  action -> setStatusText (i18n("Shows/hides the brightness toolbar"));
  action = new KToggleAction (i18n("Show &Contrast Toolbar"), "contrast", 0, this, SLOT (onViewContrastToolbar()), actionCollection(), "settings_contrast_toolbar");
  action -> setStatusText (i18n("Shows/hides the contrast toolbar"));
  action = new KToggleAction (i18n("Show &Hue Toolbar"), "hue", 0, this, SLOT (onViewHueToolbar()), actionCollection(), "settings_hue_toolbar");
  action -> setStatusText (i18n("Shows/hides the hue toolbar"));
  action = new KToggleAction (i18n("Show &Saturation Toolbar"), "saturation", 0, this, SLOT (onViewSaturationToolbar()), actionCollection(), "settings_saturation_toolbar");
  action -> setStatusText (i18n("Shows/hides the saturation toolbar"));
  action = KStdAction::showStatusbar (this, SLOT (onViewStatusbar()), actionCollection());
  action -> setStatusText (i18n("Shows/hides the status bar"));

  m_updating = true;
  KPlayerSliderAction* sa = new KPlayerSliderAction (i18n("Progress"), this, SLOT (onProgressChanged (int)), actionCollection(), "player_progress");
  sa -> slider() -> setup (0, 0, 0, 0, 0);
//sa -> slider() -> setTracking (false);
  sa -> setStatusText (i18n("Shows player progress and allows seeking"));
  sa = new KPlayerSliderAction (i18n("Volume"), this, SLOT (onVolumeChanged (int)), actionCollection(), "player_volume");
  sa -> slider() -> setup (0, 100, 50);
  sa -> setStatusText (i18n("Changes volume level"));
  sa = new KPlayerSliderAction (i18n("Contrast"), this, SLOT (onContrastChanged (int)), actionCollection(), "video_contrast");
  sa -> slider() -> setup (-100, 100, 0, 20);
  sa -> setStatusText (i18n("Changes contrast level"));
  sa = new KPlayerSliderAction (i18n("Brightness"), this, SLOT (onBrightnessChanged (int)), actionCollection(), "video_brightness");
  sa -> slider() -> setup (-50, 50, 0);
  sa -> setStatusText (i18n("Changes brightness level"));
  sa = new KPlayerSliderAction (i18n("Hue"), this, SLOT (onHueChanged (int)), actionCollection(), "video_hue");
  sa -> slider() -> setup (-100, 100, 0, 20);
  sa -> setStatusText (i18n("Changes hue level"));
  sa = new KPlayerSliderAction (i18n("Saturation"), this, SLOT (onSaturationChanged (int)), actionCollection(), "video_saturation");
  sa -> slider() -> setup (-100, 100, 0, 20);
  sa -> setStatusText (i18n("Changes saturation level"));

  KPlayerPopupSliderAction* psa = new KPlayerPopupSliderAction (i18n("&Volume Popup"), "volume", CTRL + Key_V, this, SLOT (onVolumeChanged (int)), actionCollection(), "popup_volume");
  psa -> slider() -> setup (0, 100, 50);
  psa -> setStatusText (i18n("Shows the volume popup slider"));
  psa = new KPlayerPopupSliderAction (i18n("&Contrast Popup"), "contrast", CTRL + Key_C, this, SLOT (onContrastChanged (int)), actionCollection(), "popup_contrast");
  psa -> slider() -> setup (-100, 100, 0, 20);
  psa -> setStatusText (i18n("Shows the contrast popup slider"));
  psa = new KPlayerPopupSliderAction (i18n("&Brightness Popup"), "brightness", CTRL + Key_B, this, SLOT (onBrightnessChanged (int)), actionCollection(), "popup_brightness");
  psa -> slider() -> setup (-50, 50, 0);
  psa -> setStatusText (i18n("Shows the brightness popup slider"));
  psa = new KPlayerPopupSliderAction (i18n("&Hue Popup"), "hue", CTRL + Key_H, this, SLOT (onHueChanged (int)), actionCollection(), "popup_hue");
  psa -> slider() -> setup (-100, 100, 0, 20);
  psa -> setStatusText (i18n("Shows the hue popup slider"));
  psa = new KPlayerPopupSliderAction (i18n("&Saturation Popup"), "saturation", CTRL + Key_J, this, SLOT (onSaturationChanged (int)), actionCollection(), "popup_saturation");
  psa -> slider() -> setup (-100, 100, 0, 20);
  psa -> setStatusText (i18n("Shows the saturation popup slider"));
  m_updating = false;

  createGUI();
}

void KPlayer::initStatusBar (void)
{
  statusBar() -> insertItem (i18n("Ready"), ID_STATUS_MSG, 1);
  statusBar() -> insertItem (i18n("Idle"), ID_STATE_MSG, 1, true);
  statusBar() -> insertItem ("", ID_PROGRESS_MSG, 1, true);
  QObjectList* children = (QObjectList*) statusBar() -> children();
  for ( uint i = 0; i < children -> count(); i ++ )
  {
    QObject* child = children -> at (i);
    if ( child -> inherits ("KStatusBarLabel") )
    {
      KStatusBarLabel* label = (KStatusBarLabel*) child;
      if ( label -> text() == i18n("Ready") )
        m_status_label = label;
      else if ( label -> text() == i18n("Idle") )
        m_state_label = label;
      else if ( label -> text().isEmpty() )
        m_progress_label = label;
    }
  }
}

void KPlayer::saveOptions (void)
{
  KConfig* config = kapp -> config();
  saveMainWindowSettings (config, "General Options");
  config -> writeEntry ("Menu Bar Normally Visible", m_menubar_normally_visible);
  config -> writeEntry ("Menu Bar FullScreen Visible", m_menubar_fullscreen_visible);
  for ( int i = 0; i < KPLAYER_TOOLBARS; i ++ )
  {
    kdDebug() << m_toolbar[i].name << ": " << m_toolbar[i].normally_visible << " " << m_toolbar[i].fullscreen_visible << "\n";
    config -> writeEntry (QString (m_toolbar[i].name) + " Normally Visible", m_toolbar[i].normally_visible);
    config -> writeEntry (QString (m_toolbar[i].name) + " FullScreen Visible", m_toolbar[i].fullscreen_visible);
  }
  config -> writeEntry ("Status Bar Normally Visible", m_statusbar_normally_visible);
  config -> writeEntry ("Status Bar FullScreen Visible", m_statusbar_fullscreen_visible);
  recentFilesAction() -> saveEntries (config, "Recent Files");
  config -> setGroup ("Player Options");
  config -> writeEntry ("Maintain Aspect", kPlayerWidget() -> originalAspect());
  config -> writeEntry ("Frame Dropping", kPlayerProcess() -> frameDropping());
  config -> writeEntry ("Volume", kPlayerProcess() -> volume());
  config -> writeEntry ("Mute", toggleAction ("player_mute") -> isChecked());
  config -> writeEntry ("Contrast", kPlayerProcess() -> contrast());
  config -> writeEntry ("Brightness", kPlayerProcess() -> brightness());
  config -> writeEntry ("Hue", kPlayerProcess() -> hue());
  config -> writeEntry ("Saturation", kPlayerProcess() -> saturation());
  config -> writeEntry ("Full Screen", isFullScreen());
}

void KPlayer::readOptions (void)
{
  KConfig* config = kapp -> config();
  applyMainWindowSettings (config, "General Options");
  m_menubar_normally_visible = config -> readBoolEntry ("Menu Bar Normally Visible", m_menubar_normally_visible);
  m_menubar_fullscreen_visible = config -> readBoolEntry ("Menu Bar FullScreen Visible", m_menubar_fullscreen_visible);
  toggleAction (KStdAction::stdName (KStdAction::ShowMenubar)) -> setChecked (m_menubar_normally_visible);
  if ( m_menubar_normally_visible )
    menuBar() -> show();
  else
    menuBar() -> hide();
  for ( int i = 0; i < KPLAYER_TOOLBARS; i ++ )
  {
    m_toolbar[i].normally_visible = config -> readBoolEntry (QString (m_toolbar[i].name) + " Normally Visible", m_toolbar[i].normally_visible);
    m_toolbar[i].fullscreen_visible = config -> readBoolEntry (QString (m_toolbar[i].name) + " FullScreen Visible", m_toolbar[i].fullscreen_visible);
    toggleAction (m_toolbar[i].action) -> setChecked (m_toolbar[i].normally_visible);
    if ( m_toolbar[i].normally_visible )
      toolBar (m_toolbar[i].name) -> show();
    else
      toolBar (m_toolbar[i].name) -> hide();
    kdDebug() << m_toolbar[i].name << ": " << m_toolbar[i].normally_visible << " " << m_toolbar[i].fullscreen_visible << "\n";
  }
  m_statusbar_normally_visible = config -> readBoolEntry ("Status Bar Normally Visible", m_statusbar_normally_visible);
  m_statusbar_fullscreen_visible = config -> readBoolEntry ("Status Bar FullScreen Visible", m_statusbar_fullscreen_visible);
  toggleAction (KStdAction::stdName (KStdAction::ShowStatusbar)) -> setChecked (m_statusbar_normally_visible);
  if ( m_statusbar_normally_visible )
    statusBar() -> show();
  else
    statusBar() -> hide();
  recentFilesAction() -> loadEntries (kapp -> config(), "Recent Files");
  config -> setGroup ("Player Options");
  kPlayerWidget() -> maintainOriginalAspect (config -> readBoolEntry ("Maintain Aspect", false));
  if ( kPlayerWidget() -> originalAspect() )
    toggleAction ("view_original_aspect") -> setChecked (true);
  int numEntry = config -> readNumEntry ("Frame Dropping", 0);
//kPlayerProcess() -> setFrameDropping ((KPlayerProcess::FrameDropping) numEntry);
  if ( numEntry == KPlayerProcess::Soft )
    toggleAction ("player_soft_frame_drop") -> activate();
  else if ( numEntry == KPlayerProcess::Hard )
    toggleAction ("player_hard_frame_drop") -> activate();
  setVolume (config -> readNumEntry ("Volume", 50));
  if ( config -> readBoolEntry ("Mute", false) )
    toggleAction ("player_mute") -> activate();
  numEntry = config -> readNumEntry ("Contrast", 0);
  setContrast (numEntry);
  setBrightness (config -> readNumEntry ("Brightness", 0));
  setHue (config -> readNumEntry ("Hue", 0));
  setSaturation (config -> readNumEntry ("Saturation", 0));
  m_initial_fullscreen = config -> readBoolEntry ("Full Screen", false);
}

void KPlayer::saveProperties (KConfig* config) const
{
  config -> writeEntry ("File URL", kPlayerProcess() -> url());
  config -> writeEntry ("Subtitle URL", kPlayerProcess() -> subtitleUrl());
}

void KPlayer::readProperties (KConfig* config)
{
  QString name = config -> readEntry ("File URL", "");
  if ( ! name.isEmpty() )
    kPlayerProcess() -> setUrl (name);
  name = config -> readEntry ("Subtitle URL", "");
  if ( ! name.isEmpty() )
    kPlayerProcess() -> setSubtitleUrl (name);
}

void KPlayer::enableToolbar (int index)
{
  Toolbar& toolbar = m_toolbar [index];
  KToggleAction* action = toggleAction (toolbar.action);
  if ( action -> isEnabled() )
    return;
  action -> setEnabled (true);
  if ( isFullScreen() ? toolbar.fullscreen_visible : toolbar.normally_visible )
  {
    action -> setChecked (true);
    toolBar (toolbar.name) -> show();
  }
  kdDebug() << "Enabled " << toolbar.name << "\n";
}

void KPlayer::disableToolbar (int index)
{
  Toolbar& toolbar = m_toolbar [index];
  KToggleAction* action = toggleAction (toolbar.action);
  if ( ! action -> isEnabled() )
    return;
  if ( action -> isChecked() )
  {
    action -> setChecked (false);
    toolBar (toolbar.name) -> hide();
  }
  action -> setEnabled (false);
  kdDebug() << "Disabled " << toolbar.name << "\n";
}

void KPlayer::enablePlayerActions (void)
{
  KPlayerProcess::State state = kPlayerProcess() -> state();
  action ("player_play") -> setEnabled (! kPlayerProcess() -> url(). isEmpty()
    && state != KPlayerProcess::Running && state != KPlayerProcess::Playing);
  action ("player_pause") -> setEnabled (state != KPlayerProcess::Idle);
  bool busy = state != KPlayerProcess::Idle;
  sliderAction ("player_progress") -> slider() -> setEnabled (busy);
  action ("player_stop") -> setEnabled (busy);
  action ("player_forward") -> setEnabled (busy);
  action ("player_fast_forward") -> setEnabled (busy);
  action ("player_backward") -> setEnabled (busy);
  action ("player_fast_backward") -> setEnabled (busy);
  action ("player_start") -> setEnabled (busy);
  if ( kPlayerProcess() -> length() != 0 )
    enableToolbar (1);
  else
    disableToolbar (1);
}

void KPlayer::enableVideoActions (void)
{
  bool video = ! kPlayerProcess() -> size(). isEmpty();
  action ("file_open_subtitle") -> setEnabled (video);
  action ("file_open_subtitle_url") -> setEnabled (video);
  action ("file_unload_subtitles") -> setEnabled (video);
  action ("view_full_screen") -> setEnabled (video || isFullScreen());
  action ("view_original_aspect") -> setEnabled (video);
  action ("view_current_aspect") -> setEnabled (video);
  action ("view_aspect_4_3") -> setEnabled (video);
  action ("view_aspect_16_9") -> setEnabled (video);
  action ("video_brightness_up") -> setEnabled (video);
  action ("video_brightness_down") -> setEnabled (video);
  action ("video_contrast_up") -> setEnabled (video);
  action ("video_contrast_down") -> setEnabled (video);
  action ("video_hue_up") -> setEnabled (video);
  action ("video_hue_down") -> setEnabled (video);
  action ("video_saturation_up") -> setEnabled (video);
  action ("video_saturation_down") -> setEnabled (video);
  for ( int i = FIRST_VIDEO_TOOLBAR; i < KPLAYER_TOOLBARS; i ++ )
    if ( video )
      enableToolbar (i);
    else
      disableToolbar (i);
  enableZoomActions();
}

void KPlayer::enableZoomActions (void)
{
  toggleAction ("view_zoom_1_2") -> setChecked (kPlayerWidget() -> isZoomFactor (1, 2));
  toggleAction ("view_zoom_1_1") -> setChecked (kPlayerWidget() -> isZoomFactor (1));
  toggleAction ("view_zoom_3_2") -> setChecked (kPlayerWidget() -> isZoomFactor (3, 2));
  toggleAction ("view_zoom_2_1") -> setChecked (kPlayerWidget() -> isZoomFactor (2));
  toggleAction ("view_zoom_5_2") -> setChecked (kPlayerWidget() -> isZoomFactor (5, 2));
  toggleAction ("view_zoom_3_1") -> setChecked (kPlayerWidget() -> isZoomFactor (3));
  bool canZoom = ! isFullScreen() && ! kPlayerProcess() -> size(). isEmpty();
  toggleAction ("view_zoom_in") -> setEnabled (canZoom);
  toggleAction ("view_zoom_out") -> setEnabled (canZoom);
  toggleAction ("view_zoom_1_2") -> setEnabled (canZoom);
  toggleAction ("view_zoom_1_1") -> setEnabled (canZoom);
  toggleAction ("view_zoom_3_2") -> setEnabled (canZoom);
  toggleAction ("view_zoom_2_1") -> setEnabled (canZoom);
  toggleAction ("view_zoom_5_2") -> setEnabled (canZoom);
  toggleAction ("view_zoom_3_1") -> setEnabled (canZoom);
}

/*void KPlayer::checkBarActions (void)
{
  toggleAction (KStdAction::stdName (KStdAction::ShowMenubar)) -> setChecked (menuBar() -> isVisible());
  for ( int i = 0; i < KPLAYER_TOOLBARS; i ++ )
    toggleAction (m_toolbar[i].action) -> setChecked (toolBar (m_toolbar[i].name) -> isVisible());
  toggleAction (KStdAction::stdName (KStdAction::ShowStatusbar)) -> setChecked (statusBar() -> isVisible());
}*/

void KPlayer::start (void)
{
  if ( isMinimized() )
  {
    kdDebug() << "Restoring minimized window\n";
    clearWState (WState_Visible); // or showNormal won't work
    showNormal();
  }
  else
    show();
  raise();
  setActiveWindow(); // doesn't work after restoring from minimized state
  kdDebug() << "Main window minimum size " << minimumWidth() << "x" << minimumHeight()
    << ", maximum size " << maximumWidth() << "x" << maximumHeight()
    << ", size " << width() << "x" << height() << "\n";
  KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
  if ( args -> count() == 1 )
  {
    kPlayerProcess() -> setUrl (args -> arg (args -> count() - 1));
    if ( m_initial_fullscreen )
    {
      kdDebug() << "Initial full screen\n";
      toggleAction ("view_full_screen") -> activate();
      m_initial_fullscreen = false;
    }
    play();
  }
//else
//  onFileOpen();
  args -> clear();
}

//bool KPlayerX11TestGrab (Display*, int);

/*bool KPlayer::event (QEvent* ev)
{
//kdDebug() << "KPlayer event: type " << ev -> type()
//  << " spontaneous " << ev -> spontaneous() << "\n";
//if ( ev -> type() == QEvent::LayoutHint && isFullScreen() )
//  return false;
  return KMainWindow::event (ev);
}*/

void KPlayer::contextMenuEvent (QContextMenuEvent* event)
{
  QPopupMenu* popup = (QPopupMenu*) factory() -> container ("player_popup", this);
  popup -> popup (event -> globalPos());
  event -> accept();
}

void KPlayer::closeEvent (QCloseEvent* event)
{
//if ( isFullScreen() )
//  action ("view_full_screen") -> activate();
  saveOptions();
  kPlayerProcess() -> stop();
  kPlayerProcess() -> kill();
  kPlayerWidget() -> close (true);
  KMainWindow::closeEvent (event);
}

void KPlayer::onFileOpen (void)
{
  if ( kPlayerWidget() -> openFile() )
    play();
}

void KPlayer::onFileOpenUrl (void)
{
  if ( kPlayerWidget() -> openUrl() )
    play();
}

void KPlayer::onFileOpenRecent (const KURL& url)
{
  kPlayerProcess() -> setUrl (url.prettyURL (0, KURL::StripFileProtocol));
  play();
}

void KPlayer::onFileOpenSubtitle (void)
{
  if ( kPlayerWidget() -> openSubtitle() )
  {
    kPlayerProcess() -> setAutoloadSubtitles (true);
    toggleAction ("file_unload_subtitles") -> setChecked (false);
    if ( kPlayerProcess() -> state() != KPlayerProcess::Idle )
    {
      kPlayerProcess() -> stop();
      kPlayerProcess() -> play();
    }
  }
}

void KPlayer::onFileOpenSubtitleUrl (void)
{
  if ( kPlayerWidget() -> openSubtitleUrl() )
  {
    kPlayerProcess() -> setAutoloadSubtitles (true);
    toggleAction ("file_unload_subtitles") -> setChecked (false);
    if ( kPlayerProcess() -> state() != KPlayerProcess::Idle )
    {
      kPlayerProcess() -> stop();
      kPlayerProcess() -> play();
    }
  }
}

void KPlayer::onFileUnloadSubtitles (void)
{
  if ( toggleAction ("file_unload_subtitles") -> isChecked() )
  {
    kPlayerProcess() -> setSubtitleUrl ("");
    kPlayerProcess() -> setAutoloadSubtitles (false);
  }
  else
    kPlayerProcess() -> setAutoloadSubtitles (true);
  if ( kPlayerProcess() -> state() != KPlayerProcess::Idle )
  {
    kPlayerProcess() -> stop();
    kPlayerProcess() -> play();
  }
}

void KPlayer::onFileQuit (void)
{
  close();
}

void KPlayer::onViewFullScreen (void)
{
  m_resizing = true;
  KToggleAction* action;
  if ( toggleAction ("view_full_screen") -> isChecked() )
  {
    action = toggleAction (KStdAction::stdName (KStdAction::ShowMenubar));
    action -> setChecked (m_menubar_fullscreen_visible);
    if ( m_menubar_fullscreen_visible && ! m_menubar_normally_visible )
      menuBar() -> show();
    else if ( ! m_menubar_fullscreen_visible && m_menubar_normally_visible )
      menuBar() -> hide();
    for ( int i = 0; i < KPLAYER_TOOLBARS; i ++ )
    {
      action = toggleAction (m_toolbar[i].action);
      action -> setChecked (m_toolbar[i].fullscreen_visible);
      if ( m_toolbar[i].fullscreen_visible && ! m_toolbar[i].normally_visible )
        toolBar (m_toolbar[i].name) -> show();
      else if ( ! m_toolbar[i].fullscreen_visible && m_toolbar[i].normally_visible )
        toolBar (m_toolbar[i].name) -> hide();
    }
    action = toggleAction (KStdAction::stdName (KStdAction::ShowStatusbar));
    action -> setChecked (m_statusbar_fullscreen_visible);
    if ( m_statusbar_fullscreen_visible && ! m_statusbar_normally_visible )
      statusBar() -> show();
    else if ( ! m_statusbar_fullscreen_visible && m_statusbar_normally_visible )
      statusBar() -> hide();
    kPlayerWorkspace() -> setFreeResizeMode (false);
    m_resizing = false;
    showFullScreen();
    kdDebug() << "Full screen geometry: " << x() << "x" << y() << " " << width() << "x" << height() << "\n";
    kdDebug() << "Full screen geometry: " << frameGeometry().x() << "x" << frameGeometry().y() << " " << frameGeometry().width() << "x" << frameGeometry().height() << "\n";
    setGeometry (0, -1, width(), height() + 2);
    kdDebug() << "Full screen geometry: " << x() << "x" << y() << " " << width() << "x" << height() << "\n";
    kdDebug() << "Full screen geometry: " << frameGeometry().x() << "x" << frameGeometry().y() << " " << frameGeometry().width() << "x" << frameGeometry().height() << "\n";
  }
  else
  {
    action = toggleAction (KStdAction::stdName (KStdAction::ShowMenubar));
    action -> setChecked (m_menubar_normally_visible);
    if ( m_menubar_normally_visible && ! m_menubar_fullscreen_visible )
      menuBar() -> show();
    else if ( ! m_menubar_normally_visible && m_menubar_fullscreen_visible )
      menuBar() -> hide();
    for ( int i = 0; i < KPLAYER_TOOLBARS; i ++ )
    {
      action = toggleAction (m_toolbar[i].action);
      action -> setChecked (m_toolbar[i].normally_visible);
      if ( m_toolbar[i].normally_visible && ! m_toolbar[i].fullscreen_visible )
        toolBar (m_toolbar[i].name) -> show();
      else if ( ! m_toolbar[i].normally_visible && m_toolbar[i].fullscreen_visible )
        toolBar (m_toolbar[i].name) -> hide();
    }
    action = toggleAction (KStdAction::stdName (KStdAction::ShowStatusbar));
    action -> setChecked (m_statusbar_normally_visible);
    if ( m_statusbar_normally_visible && ! m_statusbar_fullscreen_visible )
      statusBar() -> show();
    else if ( ! m_statusbar_normally_visible && m_statusbar_fullscreen_visible )
      statusBar() -> hide();
    kPlayerWorkspace() -> setFreeResizeMode (true);
    m_resizing = false;
    showNormal();
  }
}

void KPlayer::onViewZoomIn (void)
{
  kPlayerWidget() -> zoomIn();
  zoom();
}

void KPlayer::onViewZoomOut (void)
{
  kPlayerWidget() -> zoomOut();
  zoom();
}

void KPlayer::onViewZoom12 (void)
{
  kPlayerWidget() -> setZoomFactor (1, 2);
  zoom();
}

void KPlayer::onViewZoom11 (void)
{
  kPlayerWidget() -> setZoomFactor (1);
  zoom();
}

void KPlayer::onViewZoom32 (void)
{
  kPlayerWidget() -> setZoomFactor (3, 2);
  zoom();
}

void KPlayer::onViewZoom21 (void)
{
  kPlayerWidget() -> setZoomFactor (2);
  zoom();
}

void KPlayer::onViewZoom52 (void)
{
  kPlayerWidget() -> setZoomFactor (5, 2);
  zoom();
}

void KPlayer::onViewZoom31 (void)
{
  kPlayerWidget() -> setZoomFactor (3);
  zoom();
}

void KPlayer::onViewMaintainOriginalAspect (void)
{
  kPlayerWidget() -> maintainOriginalAspect (toggleAction ("view_original_aspect") -> isChecked());
  zoom();
}

void KPlayer::onViewMaintainCurrentAspect (void)
{
  kPlayerWidget() -> maintainCurrentAspect (toggleAction ("view_current_aspect") -> isChecked());
  zoom();
}

void KPlayer::onViewAspect43 (void)
{
  if ( toggleAction ("view_aspect_4_3") -> isChecked() )
    kPlayerWidget() -> setFixedAspect (4, 3);
  else
    kPlayerWidget() -> setFixedAspect (0, 0);
  zoom();
}

void KPlayer::onViewAspect169 (void)
{
  if ( toggleAction ("view_aspect_16_9") -> isChecked() )
    kPlayerWidget() -> setFixedAspect (16, 9);
  else
    kPlayerWidget() -> setFixedAspect (0, 0);
  zoom();
}

void KPlayer::play (void)
{
  kPlayerProcess() -> stop();
  KToggleAction* action = toggleAction ("view_current_aspect");
  if ( action -> isChecked() )
  {
    action -> setChecked (false);
    kPlayerWidget() -> maintainCurrentAspect (false);
  }
  action = toggleAction ("view_aspect_4_3");
  if ( action -> isChecked() )
  {
    action -> setChecked (false);
    kPlayerWidget() -> setFixedAspect (0, 0);
  }
  action = toggleAction ("view_aspect_16_9");
  if ( action -> isChecked() )
  {
    action -> setChecked (false);
    kPlayerWidget() -> setFixedAspect (0, 0);
  }
  if ( m_progress_label )
    m_progress_label -> setText ("");
//kPlayerProcess() -> setVolume (sliderAction ("player_volume") -> slider() -> value());
  setCaption (kPlayerProcess() -> url());
  kPlayerProcess() -> get_info();
  kPlayerProcess() -> play();
  KRecentFilesAction* recent = recentFilesAction();
  recent -> setEnabled (true);
  recent -> addURL (KCmdLineArgs::makeURL (kPlayerProcess() -> url()));
  recent -> setCurrentItem (0);
}

void KPlayer::onPlayerPlay (void)
{
  if ( kPlayerProcess() -> state() == KPlayerProcess::Paused )
  {
    kPlayerProcess() -> pause();
    return;
  }
  if ( kPlayerProcess() -> state() != KPlayerProcess::Idle )
    return;
  kPlayerProcess() -> play();
}

void KPlayer::onPlayerPause (void)
{
  if ( kPlayerProcess() -> state() != KPlayerProcess::Idle )
    kPlayerProcess() -> pause();
  else
  {
    KToggleAction* action = toggleAction ("player_pause");
    if ( action -> isChecked() )
      action -> setChecked (false);
  }
}

void KPlayer::onPlayerStop (void)
{
  kPlayerProcess() -> stop();
}

void KPlayer::onPlayerForward (void)
{
  int seek = kPlayerProcess() -> length() == 0 ? 10 :
    int (roundf (kPlayerProcess() -> length() / 100));
  if ( seek == 0 )
    seek = 1;
  kPlayerProcess() -> relativeSeek (seek);
}

void KPlayer::onPlayerFastForward (void)
{
  int seek = kPlayerProcess() -> length() == 0 ? 60 :
    int (roundf (kPlayerProcess() -> length() / 10));
  if ( seek == 0 )
    seek = 2;
  kPlayerProcess() -> relativeSeek (seek);
}

void KPlayer::onPlayerBackward (void)
{
  int seek = kPlayerProcess() -> length() == 0 ? 10 :
    int (roundf (kPlayerProcess() -> length() / 100));
  if ( seek == 0 )
    seek = 1;
  kPlayerProcess() -> relativeSeek (- seek);
}

void KPlayer::onPlayerFastBackward (void)
{
  int seek = kPlayerProcess() -> length() == 0 ? 60 :
    int (roundf (kPlayerProcess() -> length() / 10));
  if ( seek == 0 )
    seek = 2;
  kPlayerProcess() -> relativeSeek (- seek);
}

void KPlayer::onPlayerStart (void)
{
  kPlayerProcess() -> absoluteSeek (0);
}

void KPlayer::setVolume (int volume)
{
  m_updating = true;
  if ( volume != kPlayerProcess() -> volume() )
    kPlayerProcess() -> setVolume (volume);
  KPlayerSlider* slider = sliderAction ("player_volume") -> slider();
  if ( volume != slider -> value() )
    slider -> setValue (volume);
  slider = popupAction ("popup_volume") -> slider();
  if ( volume != slider -> value() )
    slider -> setValue (volume);
  m_updating = false;
}

void KPlayer::setContrast (int contrast)
{
  m_updating = true;
  if ( contrast != kPlayerProcess() -> contrast() )
    kPlayerProcess() -> setContrast (contrast);
  KPlayerSlider* slider = sliderAction ("video_contrast") -> slider();
  if ( contrast != slider -> value() )
    slider -> setValue (contrast);
  slider = popupAction ("popup_contrast") -> slider();
  if ( contrast != slider -> value() )
    slider -> setValue (contrast);
  m_updating = false;
}

void KPlayer::setBrightness (int brightness)
{
  m_updating = true;
  if ( brightness != kPlayerProcess() -> brightness() )
    kPlayerProcess() -> setBrightness (brightness);
  KPlayerSlider* slider = sliderAction ("video_brightness") -> slider();
  if ( brightness != slider -> value() )
    slider -> setValue (brightness);
  slider = popupAction ("popup_brightness") -> slider();
  if ( brightness != slider -> value() )
    slider -> setValue (brightness);
  m_updating = false;
}

void KPlayer::setHue (int hue)
{
  m_updating = true;
  if ( hue != kPlayerProcess() -> hue() )
    kPlayerProcess() -> setHue (hue);
  KPlayerSlider* slider = sliderAction ("video_hue") -> slider();
  if ( hue != slider -> value() )
    slider -> setValue (hue);
  slider = popupAction ("popup_hue") -> slider();
  if ( hue != slider -> value() )
    slider -> setValue (hue);
  m_updating = false;
}

void KPlayer::setSaturation (int saturation)
{
  m_updating = true;
  if ( saturation != kPlayerProcess() -> saturation() )
    kPlayerProcess() -> setSaturation (saturation);
  KPlayerSlider* slider = sliderAction ("video_saturation") -> slider();
  if ( saturation != slider -> value() )
    slider -> setValue (saturation);
  slider = popupAction ("popup_saturation") -> slider();
  if ( saturation != slider -> value() )
    slider -> setValue (saturation);
  m_updating = false;
}

void KPlayer::onPlayerVolumeUp (void)
{
  if ( kPlayerProcess() -> volume() < 100 )
    setVolume (kPlayerProcess() -> volume() + 1);
}

void KPlayer::onPlayerVolumeDown (void)
{
  if ( kPlayerProcess() -> volume() > 0 )
    setVolume (kPlayerProcess() -> volume() - 1);
}

void KPlayer::onPlayerMute (void)
{
  kPlayerProcess() -> mute();
/*if ( kPlayerProcess() -> state() != KPlayerProcess::Idle )
    kPlayerProcess() -> mute();
  else
  {
    KToggleAction* action = toggleAction ("player_mute");
    if ( action -> isChecked() )
      action -> setChecked (false);
  }*/
}

void KPlayer::onPlayerSoftFrameDrop (void)
{
  kPlayerProcess() -> setFrameDropping (toggleAction ("player_soft_frame_drop") -> isChecked() ?
    KPlayerProcess::Soft : KPlayerProcess::None);
}

void KPlayer::onPlayerHardFrameDrop (void)
{
  kPlayerProcess() -> setFrameDropping (toggleAction ("player_hard_frame_drop") -> isChecked() ?
    KPlayerProcess::Hard : KPlayerProcess::None);
}

void KPlayer::onVideoBrightnessUp (void)
{
  if ( kPlayerProcess() -> brightness() < 100 )
    setBrightness (kPlayerProcess() -> brightness() + 1);
}

void KPlayer::onVideoBrightnessDown (void)
{
  if ( kPlayerProcess() -> brightness() > -100 )
    setBrightness (kPlayerProcess() -> brightness() - 1);
}

void KPlayer::onVideoContrastUp (void)
{
  if ( kPlayerProcess() -> contrast() < 100 )
    setContrast (kPlayerProcess() -> contrast() + 1);
}

void KPlayer::onVideoContrastDown (void)
{
  if ( kPlayerProcess() -> contrast() > -100 )
    setContrast (kPlayerProcess() -> contrast() - 1);
}

void KPlayer::onVideoHueUp (void)
{
  if ( kPlayerProcess() -> hue() < 100 )
    setHue (kPlayerProcess() -> hue() + 1);
}

void KPlayer::onVideoHueDown (void)
{
  if ( kPlayerProcess() -> hue() > -100 )
    setHue (kPlayerProcess() -> hue() - 1);
}

void KPlayer::onVideoSaturationUp (void)
{
  if ( kPlayerProcess() -> saturation() < 100 )
    setSaturation (kPlayerProcess() -> saturation() + 1);
}

void KPlayer::onVideoSaturationDown (void)
{
  if ( kPlayerProcess() -> saturation() > -100 )
    setSaturation (kPlayerProcess() -> saturation() - 1);
}

void KPlayer::onViewMenubar (void)
{
  bool visible = toggleAction (KStdAction::stdName (KStdAction::ShowMenubar)) -> isChecked();
  if ( isFullScreen() )
    m_menubar_fullscreen_visible = visible;
  else
    m_menubar_normally_visible = visible;
  if ( visible )
    menuBar() -> show();
  else
    menuBar() -> hide();
}

void KPlayer::onViewMainToolbar (void)
{
  bool visible = toggleAction (KStdAction::stdName (KStdAction::ShowToolbar)) -> isChecked();
  if ( isFullScreen() )
    m_toolbar[0].fullscreen_visible = visible;
  else
    m_toolbar[0].normally_visible = visible;
  if ( visible )
    toolBar ("mainToolBar") -> show();
  else
    toolBar ("mainToolBar") -> hide();
}

void KPlayer::onViewProgressToolbar (void)
{
  bool visible = toggleAction ("settings_progress_toolbar") -> isChecked();
  if ( isFullScreen() )
    m_toolbar[1].fullscreen_visible = visible;
  else
    m_toolbar[1].normally_visible = visible;
  if ( visible )
    toolBar ("progressToolBar") -> show();
  else
    toolBar ("progressToolBar") -> hide();
}

void KPlayer::onViewVolumeToolbar (void)
{
  bool visible = toggleAction ("settings_volume_toolbar") -> isChecked();
  if ( isFullScreen() )
    m_toolbar[2].fullscreen_visible = visible;
  else
    m_toolbar[2].normally_visible = visible;
  if ( visible )
    toolBar ("volumeToolBar") -> show();
  else
    toolBar ("volumeToolBar") -> hide();
}

void KPlayer::onViewBrightnessToolbar (void)
{
  bool visible = toggleAction ("settings_brightness_toolbar") -> isChecked();
  if ( isFullScreen() )
    m_toolbar[3].fullscreen_visible = visible;
  else
    m_toolbar[3].normally_visible = visible;
  if ( visible )
    toolBar ("brightnessToolBar") -> show();
  else
    toolBar ("brightnessToolBar") -> hide();
}

void KPlayer::onViewContrastToolbar (void)
{
  bool visible = toggleAction ("settings_contrast_toolbar") -> isChecked();
  if ( isFullScreen() )
    m_toolbar[4].fullscreen_visible = visible;
  else
    m_toolbar[4].normally_visible = visible;
  if ( visible )
    toolBar ("contrastToolBar") -> show();
  else
    toolBar ("contrastToolBar") -> hide();
}

void KPlayer::onViewHueToolbar (void)
{
  bool visible = toggleAction ("settings_hue_toolbar") -> isChecked();
  if ( isFullScreen() )
    m_toolbar[5].fullscreen_visible = visible;
  else
    m_toolbar[5].normally_visible = visible;
  if ( visible )
    toolBar ("hueToolBar") -> show();
  else
    toolBar ("hueToolBar") -> hide();
}

void KPlayer::onViewSaturationToolbar (void)
{
  bool visible = toggleAction ("settings_saturation_toolbar") -> isChecked();
  if ( isFullScreen() )
    m_toolbar[6].fullscreen_visible = visible;
  else
    m_toolbar[6].normally_visible = visible;
  if ( visible )
    toolBar ("saturationToolBar") -> show();
  else
    toolBar ("saturationToolBar") -> hide();
}

void KPlayer::onViewStatusbar (void)
{
  bool visible = toggleAction (KStdAction::stdName (KStdAction::ShowStatusbar)) -> isChecked();
  if ( isFullScreen() )
    m_statusbar_fullscreen_visible = visible;
  else
    m_statusbar_normally_visible = visible;
  if ( visible )
    statusBar() -> show();
  else
    statusBar() -> hide();
}

void KPlayer::onProgressChanged (int progress)
{
  if ( m_updating )
    return;
  kPlayerProcess() -> absoluteSeek ((progress + 5) / 10);
/*int seek = int (roundf (float (progress) / 10 - kPlayerProcess() -> position()));
  if ( seek == 0 )
    return;
  kPlayerProcess() -> relativeSeek (seek);*/
}

void KPlayer::onVolumeChanged (int volume)
{
  if ( ! m_updating )
    setVolume (volume);
}

void KPlayer::onBrightnessChanged (int brightness)
{
  if ( ! m_updating )
    setBrightness (brightness);
}

void KPlayer::onContrastChanged (int contrast)
{
  if ( ! m_updating )
    setContrast (contrast);
}

void KPlayer::onHueChanged (int hue)
{
  if ( ! m_updating )
    setHue (hue);
}

void KPlayer::onSaturationChanged (int saturation)
{
  if ( ! m_updating )
    setSaturation (saturation);
}

void KPlayer::onPlayerStateChanged (KPlayerProcess::State state)
{
  static char* stateMessages [4] = { "Idle", "Running", "Playing", "Paused" };
  KToggleAction* action = toggleAction ("player_pause");
  if ( state != KPlayerProcess::Paused && action -> isChecked() )
    action -> setChecked (false);
/*action = toggleAction ("player_mute");
  if ( state == KPlayerProcess::Idle && action -> isChecked() )
    action -> setChecked (false);*/
//statusBar() -> changeItem (stateMessages [state], ID_STATE_MSG);
  if ( m_state_label )
    m_state_label -> setText (stateMessages [state]);
  if ( state == KPlayerProcess::Idle )
    onPlayerProgressChanged (0);
  kdDebug() << "State change received: " << state << "\n";
  enablePlayerActions();
  enableVideoActions();
}

void KPlayer::onPlayerProgressChanged (float progress)
{
  static float prev_progress = -1;
  float diff = progress - prev_progress;
  prev_progress = progress;
  if ( diff <= 0 && diff > -0.15 && progress > 0 )
    return;
  m_updating = true;
//kdDebug() << "Progress change received: " << progress << "\n";
  KPlayerSlider* slider = sliderAction ("player_progress") -> slider();
  int maxValue = slider -> maxValue();
  if ( maxValue )
  {
    int value = int (roundf (progress * 10));
    if ( value > maxValue )
      slider -> setMaxValue (value);
    slider -> setValue (value);
  }
  float length = kPlayerProcess() -> length();
  int progressHour = int (progress) / 3600, lengthHour = int (length) / 3600;
  int progressMinute = (int (progress) - progressHour * 3600) / 60;
  int lengthMinute = (int (length) - lengthHour * 3600) / 60;
  if ( progressHour > 0 || progressMinute > 0 )
    progress -= progressHour * 3600 + progressMinute * 60;
  if ( lengthHour > 0 || lengthMinute > 0 )
    length -= lengthHour * 3600 + lengthMinute * 60;
  QString s;
  if ( length > 0 )
  {
    if ( progressHour > 0 || lengthHour > 0 )
      s.sprintf ("%u:%02u:%04.1f / %u:%02u:%04.1f", progressHour, progressMinute, progress, lengthHour, lengthMinute, length);
    else if ( progressMinute > 0 || lengthMinute > 0 )
      s.sprintf ("%u:%04.1f / %u:%04.1f", progressMinute, progress, lengthMinute, length);
    else
      s.sprintf ("%03.1f / %03.1f", progress, length);
  }
  else
  {
    if ( progressHour > 0 )
      s.sprintf ("%u:%02u:%04.1f", progressHour, progressMinute, progress);
    else if ( progressMinute > 0 )
      s.sprintf ("%u:%04.1f", progressMinute, progress);
    else
      s.sprintf ("%03.1f", progress);
  }
//statusBar() -> changeItem (s, ID_PROGRESS_MSG);
  if ( m_progress_label )
    m_progress_label -> setText (s);
  m_updating = false;
}

void KPlayer::onPlayerInfoAvailable (void)
{
  kdDebug() << "Info available. " << "Detected length: " << kPlayerProcess() -> length() << "\n";
  KPlayerSlider* slider = sliderAction ("player_progress") -> slider();
  slider -> setMaxValue (int (roundf (kPlayerProcess() -> length() * 10)));
  slider -> setPageStep (int (roundf (kPlayerProcess() -> length())));
//slider -> setLineStep (action -> pageStep());
//int width = kPlayerProcess() -> width();
//while ( width < 450 && width > 0 )
//  width += kPlayerProcess() -> width() / 2;
  kdDebug() << "Video width: " << kPlayerProcess() -> width() << "x" << kPlayerProcess() -> height() << "\n";
  if ( ! isFullScreen() && kPlayerProcess() -> size(). isEmpty() )
  {
    int i;
    for ( i = 0; i < KPLAYER_TOOLBARS; i ++ )
      if ( toolBar (m_toolbar[i].name) -> orientation() == Qt::Vertical
          && toggleAction (m_toolbar[i].action) -> isChecked() )
        break;
    if ( i == KPLAYER_TOOLBARS )
    {
      kPlayerWidget() -> setVideoSize (QSize (450, 0), true);
      zoom();
    }
/*  setMinimumHeight (100);
    kdDebug() << "Main window minimum size " << minimumWidth() << "x" << minimumHeight()
      << ", maximum size " << maximumWidth() << "x" << maximumHeight()
      << ", size " << width() << "x" << height() << "\n";
      QObjectList* cl = (QObjectList*) children();
    for ( uint i = 0; i < cl -> count(); i ++ )
      if ( cl -> at (i) -> inherits ("QWidget") )
        kdDebug() << "*** Child " << cl -> at (i) -> className() << " min height hint " <<
          (((QWidget*) cl -> at (i)) -> minimumSizeHint().height()) << "\n";
    QSize size (size());
    QApplication::sendPostedEvents();
    resize (size);*/
  }
  else if ( ! isFullScreen() )
  {
    int d = 1, n = 449 / kPlayerProcess() -> width();
    if ( n > 0 && kPlayerProcess() -> width() * n + kPlayerProcess() -> width() / 2 >= 450 )
      n *= d = 2;
    kPlayerWidget() -> setZoomFactor (n + 1, d);
    zoom();
  }
  enablePlayerActions();
  enableVideoActions();
}

void KPlayer::setStatusText (const QString& text)
{
  statusBar() -> clear();
//statusBar() -> changeItem (text, ID_STATUS_MSG);
  if ( m_status_label )
    m_status_label -> setText (text);
}

void KPlayer::setStatusMessage (const QString& text)
{
  if ( text.isEmpty() )
    return;
  statusBar() -> message (text);
  m_progress_label -> hide();
  m_state_label -> hide();
}

void KPlayer::clearStatusMessage (void)
{
  statusBar() -> clear();
  m_state_label -> show();
  m_progress_label -> show();
}

/*void KPlayer::resizeEvent (QResizeEvent* event)
{
  kdDebug() << "KPlayer::resizeEvent.\n";
  KMainWindow::resizeEvent (event);
  kdDebug() << "Main Window Size: " << width() << "x" << height()
    << ", Central Widget Size: " << centralWidget() -> width() << "x" << centralWidget() -> height() << "\n";
}*/

void KPlayer::onWidgetResized (void)
{
  if ( m_resizing )
    return;
  kdDebug() << "Widget resized event\n";
  if ( ! isFullScreen() )
  {
    if ( m_previous_size == size() && ! m_video_size.isEmpty() )
      kPlayerWidget() -> setVideoSize (m_video_size);
    else
      kPlayerWidget() -> setVideoSize (centralWidget() -> size(), true, m_previous_size.width() == width());
  }
  zoom();
}

void KPlayer::zoom (void)
{
  if ( m_resizing )
    return;
  m_resizing = true;
  bool fullScreen = isFullScreen();
  if ( fullScreen )
  {
    kPlayerWorkspace() -> setVideoSize();
    kdDebug() << "Full screen zoom done\n";
    enableZoomActions();
    m_resizing = false;
    return;
  }
  QSize video (kPlayerWidget() -> videoSize()), cwsize (centralWidget() -> size());
  m_video_size = video;
  kdDebug() << "Main Window Geometry: " << x() << "x" << y() << " " << width() << "x" << height()
    << ", Central Widget Size: " << cwsize.width() << "x" << cwsize.height()
    << ", Video Size: " << video.width() << "x" << video.height() << "\n";
  int i = 0;
  while ( cwsize != video && i ++ < 5 )
  {
    resize (size() + video - cwsize);
    QApplication::sendPostedEvents (this, QEvent::Resize);
    QApplication::sendPostedEvents (this, QEvent::LayoutHint);
    QApplication::sendPostedEvents (kPlayerWidget(), QEvent::Resize);
    QApplication::sendPostedEvents (centralWidget(), QEvent::Resize);
    cwsize = centralWidget() -> size();
    kdDebug() << "Main Window Size: " << width() << "x" << height()
      << ", Central Widget Size: " << cwsize.width() << "x" << cwsize.height() << "\n";
  }
  int xc = x() + frameGeometry().width() - QApplication::desktop() -> width();
  int yc = y() + frameGeometry().height() - QApplication::desktop() -> height();
  if ( xc > 0 || yc > 0 )
  {
    if ( xc <= 0 )
      xc = x();
    else if ( QApplication::desktop() -> width() > frameGeometry().width() )
      xc = QApplication::desktop() -> width() - frameGeometry().width();
    else
      xc = 0;
    if ( yc <= 0 )
      yc = y();
    else if ( QApplication::desktop() -> height() > frameGeometry().height() )
      yc = QApplication::desktop() -> height() - frameGeometry().height();
    else
      yc = 0;
    move (xc, yc);
  }
  m_previous_size = size();
  QApplication::postEvent (this, new QEvent (QEvent::LayoutHint));
  enableZoomActions();
//checkBarActions();
  m_resizing = false;
  kdDebug() << "Normal zoom done\n";
}

/*void KPlayer::onBarOrientationChanged (Orientation)
{
  kdDebug() << "A bar orientation has changed.\n";
  kdDebug() << "Central Widget Size: " << centralWidget() -> width() << "x" << centralWidget() -> height()
    << ", KPlayer Widget Size: " << kPlayerWidget() -> width() << "x" << kPlayerWidget() -> height() << ".\n";
}

void KPlayer::onBarPlaceChanged (QDockWindow::Place)
{
  kdDebug() << "A bar place has changed.\n";
  kdDebug() << "Central Widget Size: " << centralWidget() -> width() << "x" << centralWidget() -> height()
    << ", KPlayer Widget Size: " << kPlayerWidget() -> width() << "x" << kPlayerWidget() -> height() << ".\n";
}

void KPlayer::onBarVisibilityChanged (bool)
{
  kdDebug() << "A bar visibility has changed.\n";
  kdDebug() << "Central Widget Size: " << centralWidget() -> width() << "x" << centralWidget() -> height()
    << ", KPlayer Widget Size: " << kPlayerWidget() -> width() << "x" << kPlayerWidget() -> height() << ".\n";
}*/
