/***************************************************************************
                          kplayerprocess.cpp
                          ------------------
    begin                : Sat Jan 11 2003
    copyright            : (C) 2002-2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kprocess.h>
#include <kstandarddirs.h>
#include <qapplication.h>
#include <qregexp.h>
#include <unistd.h>

#include <kdebug.h>

#include "kplayerprocess.h"

QRegExp re_a_or_v ("^[AV]: *([0-9,]+)"), re_a_and_v ("^A: *([0-9,]+) +V: *([0-9,]+)");
QRegExp re_video ("^V(?:IDE)?O: *\\S+ +(\\d+)x(\\d+)"), re_start ("^Start playing\\.\\.\\.");
QRegExp re_playing ("^Playing "), re_exiting ("Exiting"), re_extension ("\\.[^/.]+$");
QRegExp re_ans_length ("^ANS_LENGTH=(\\d+)$");

KPlayerProcess::KPlayerProcess (void)
{
  m_player = m_helper = 0;
  m_state = Idle;
  m_ao = m_vo = 0;
  m_frame_drop = None;
  m_audio = m_video = false;
  m_width = m_height = 0;
  m_volume = 50;
  m_mute = m_paused = m_quit = false;
  m_brightness = m_contrast = m_hue = m_saturation = 0;
  m_autoload_subtitles = true;
  m_length = m_position = m_helper_position = 0;
  m_seek = m_seek_count = m_ans_length = 0;
  m_stdout_buffer_length = m_helper_buffer_length = 128;
  m_stdout_buffer = new char [m_stdout_buffer_length];
  m_helper_stdout_buffer = new char [m_helper_buffer_length];
  m_stdout_line_length = m_helper_line_length = 0;
}

KPlayerProcess::~KPlayerProcess()
{
  delete [] m_stdout_buffer;
  delete [] m_helper_stdout_buffer;
  if ( m_player )
    delete m_player;
  if ( m_helper )
    delete m_helper;
}

void KPlayerProcess::setUrl (QString url)
{
  m_url = url;
  setSubtitleUrl ("");
  if ( url[0] != '/' )
    return;
  if ( re_extension.search (url) >= 0 )
    url.replace (re_extension, ".smi");
  else
    url.append (".smi");
  kdDebug() << "Autoloading " << url << "\n";
  if ( access (url.latin1(), R_OK) == 0 )
    setSubtitleUrl (url);
  else
  {
    url.replace (re_extension, ".sub");
    kdDebug() << "Autoloading " << url << "\n";
    if ( access (url.latin1(), R_OK) == 0 )
      setSubtitleUrl (url);
  }
}

void KPlayerProcess::setState (State state)
{
  if ( m_state == state )
    return;
  m_state = state;
  emit stateChanged (state);
}

bool KPlayerProcess::get_info (void)
{
  m_ao = "null";
  m_vo = "null";
  m_audio = true;
  m_video = false;
  m_width = 0;
  m_height = 0;
  m_length = 0;
  m_seek = 0;
  m_seek_count = 0;
  m_helper_position = 0;
  m_ans_length = 0;
  m_helper = run();
  if ( ! m_helper )
    return false;
  m_helper -> writeStdin ("get_time_length\n", 16);
  return true;
}

bool KPlayerProcess::play (void)
{
  m_ao = 0;
  m_vo = "xv";
  m_audio = true;
  m_video = true;
//m_mute = false;
  m_position = 0;
  m_paused = false;
  m_quit = false;
  m_player = run();
  if ( ! m_player )
    return false;
  setState (Running);
  return true;
}

KProcess* KPlayerProcess::run (void)
{
  KProcess* player = new KProcess;
  *player << "mplayer" << "-vo" << m_vo << "-slave" << "-osdlevel" << "0";
  if ( ! m_audio )
    *player << "-nosound";
  else if ( m_ao )
    *player << "-ao" << m_ao;
  if ( m_video )
    *player << "-wid" << QCString().setNum (m_winId);
  if ( m_video )
  {
    *player << "-contrast" << QCString().setNum (m_contrast);
    *player << "-brightness" << QCString().setNum (m_brightness);
    *player << "-hue" << QCString().setNum (m_hue);
    *player << "-saturation" << QCString().setNum (m_saturation);
    if ( m_frame_drop == KPlayerProcess::Soft )
      *player << "-framedrop";
    else if ( m_frame_drop == KPlayerProcess::Hard )
      *player << "-hardframedrop";
    QString path = KGlobal::dirs() -> findResource ("appdata", "input.conf");
    if ( ! path.isEmpty() )
      *player << "-input" << "conf=" + path;
    kdDebug() << "Input.conf: " << path << "\n";
    if ( ! m_subtitle.isEmpty() )
      *player << "-sub" << m_subtitle;
    else if ( ! m_autoload_subtitles )
      *player << "-noautosub";
  }
  *player << m_url;

  QApplication::connect (player, SIGNAL (processExited (KProcess*)),
    this, SLOT (onProcessExited (KProcess*)));
  if ( m_video )
    QApplication::connect (player, SIGNAL (receivedStdout (KProcess*, char*, int)),
      this, SLOT (onReceivedStdout (KProcess*, char*, int)));
  else
    QApplication::connect (player, SIGNAL (receivedStdout (KProcess*, char*, int)),
      this, SLOT (onReceivedHelperStdout (KProcess*, char*, int)));
  QApplication::connect (player, SIGNAL (receivedStderr (KProcess*, char*, int)),
    this, SLOT (onReceivedStderr (KProcess*, char*, int)));

  if ( ! player -> start (KProcess::NotifyOnExit, KProcess::All) )
  {
    delete player;
    return 0;
  }

  return player;
}

void KPlayerProcess::pause (void)
{
  if ( ! m_player )
    return;
  m_player -> writeStdin ("pause\n", 6);
  setState (m_state == Playing ? Paused : Playing);
  kdDebug() << "pause\n";
  m_paused = ! m_paused;
}

void KPlayerProcess::stop (void)
{
  if ( ! m_player )
    return;
  m_player -> writeStdin ("quit\n", 5);
  kdDebug() << "quit\n";
  m_quit = true;
}

void KPlayerProcess::kill (void)
{
  if ( ! m_player )
    return;
  m_player -> detach();
  delete m_player;
  m_player = 0;
  setState (Idle);
}

void KPlayerProcess::absoluteSeek (int seconds)
{
  if ( ! m_player )
    return;
  if ( seconds < 0 )
    seconds = 0;
  QString s;
  s.sprintf ("seek %u 2\n", seconds);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
  m_paused = false;
}

void KPlayerProcess::relativeSeek (int seconds)
{
  if ( ! m_player || seconds == 0 )
    return;
  QString s;
  s.sprintf ("seek %d\n", seconds);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
  m_paused = false;
}

void KPlayerProcess::setVolume (int volume)
{
  if ( volume > 100 )
    volume = 100;
  else if ( volume < 0 )
    volume = 0;
  m_volume = volume;
  if ( ! m_player || m_mute )
    return;
  QString s;
  s.sprintf ("volume %d 1\n", volume);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
  m_paused = false;
}

void KPlayerProcess::mute (void)
{
  if ( m_mute )
  {
    m_mute = false;
    setVolume (m_volume);
  }
  else
  {
    if ( m_player )
      m_player -> writeStdin ("volume 0 1\n", 11);
    m_mute = true;
    m_paused = false;
  }
}

void KPlayerProcess::setFrameDropping (KPlayerProcess::FrameDropping frame_drop)
{
  m_frame_drop = frame_drop;
  if ( ! m_player )
    return;
  QString s;
  s.sprintf ("frame_drop %u\n", frame_drop);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
}

void KPlayerProcess::setBrightness (int brightness)
{
  if ( brightness > 100 )
    brightness = 100;
  else if ( brightness < -100 )
    brightness = -100;
  m_brightness = brightness;
  if ( ! m_player )
    return;
  QString s;
  s.sprintf ("brightness %d 1\n", brightness);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
}

void KPlayerProcess::setContrast (int contrast)
{
  if ( contrast > 100 )
    contrast = 100;
  else if ( contrast < -100 )
    contrast = -100;
  m_contrast = contrast;
  if ( ! m_player )
    return;
  QString s;
  s.sprintf ("contrast %d 1\n", contrast);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
}

void KPlayerProcess::setHue (int hue)
{
  if ( hue > 100 )
    hue = 100;
  else if ( hue < -100 )
    hue = -100;
  m_hue = hue;
  if ( ! m_player )
    return;
  QString s;
  s.sprintf ("hue %d 1\n", hue);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
}

void KPlayerProcess::setSaturation (int saturation)
{
  if ( saturation > 100 )
    saturation = 100;
  else if ( saturation < -100 )
    saturation = -100;
  m_saturation = saturation;
  if ( ! m_player )
    return;
  QString s;
  s.sprintf ("saturation %d 1\n", saturation);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
}

void KPlayerProcess::onProcessExited (KProcess *proc)
{
  if ( proc == m_player )
  {
    delete m_player;
    m_player = 0;
    setState (Idle);
  }
  else if ( proc == m_helper )
  {
    delete m_helper;
    m_helper = 0;
    if ( m_seek < 500 && m_helper_position >= 2 )
      m_length = m_helper_position;
    if ( m_length == 0 && m_ans_length > 0)
      m_length = m_ans_length; // better than nothing
    emit infoAvailable();
  }
  else
    delete proc;
}

float KPlayerProcess::stringToFloat (QString stime)
{
  int comma = stime.find (',');
  if ( comma >= 0 )
    stime [comma] = '.';
  return stime.toFloat();
}

void KPlayerProcess::onReceivedStdout (KProcess* proc, char* str, int len)
{
  if ( proc != m_player )
  {
    char buf [1025];
    if ( len > 1024 )
      len = 1024;
    memcpy (buf, str, len);
    buf [len] = 0;
    if ( re_exiting.search (buf) < 0 )
      proc -> writeStdin ("quit\n", 5);
    return;
  }
  bool sent = false;
  float ftime;
//kdDebug() << "stdout received length: " << len << "\n";
//kdDebug() << m_stdout_line_length << "/" << m_stdout_buffer_length << ": " << m_stdout_buffer << "\n";
  while ( len > 0 )
  {
    char* lf = (char*) memchr (str, '\n', len);
    if ( ! lf )
      lf = str + len;
    char* cr = str;
    do
    {
      cr = (char*) memchr (str, '\r', lf - str);
      if ( ! cr )
        cr = lf;
      if ( cr - str < 2 && ! *str && ! m_stdout_line_length )
        continue;
      if ( cr - str + m_stdout_line_length >= m_stdout_buffer_length )
      {
        char* old_buffer = m_stdout_buffer;
        m_stdout_buffer_length = cr - str + m_stdout_line_length + 1;
        kdDebug() << "new buffer length: " << m_stdout_buffer_length << "\n";
        m_stdout_buffer = new char [m_stdout_buffer_length];
        if ( m_stdout_line_length )
          memcpy (m_stdout_buffer, old_buffer, m_stdout_line_length);
        delete [] old_buffer;
      }
      if ( cr > str )
      {
        memcpy (m_stdout_buffer + m_stdout_line_length, str, cr - str);
        m_stdout_line_length += cr - str;
      }
      if ( cr - str == len )
      {
//      kdDebug() << "stdout return\n";
        return;
      }
      m_stdout_buffer [m_stdout_line_length] = 0;
//    if ( *m_stdout_buffer ) // && cr == lf
//    {
//      write (STDOUT_FILENO, m_stdout_buffer, m_stdout_line_length);
//      write (STDOUT_FILENO, "\n", 1);
//    }
//    kdDebug() << "Buffer: " << m_stdout_buffer << "\n";
//    kdDebug() << "matching video regex\n";
      if ( re_video.search (m_stdout_buffer) >= 0 )
      {
        m_width = re_video.cap(1).toInt();
        m_height = re_video.cap(2).toInt();
        kdDebug() << "Width: " << m_width << " Height: " << m_height << "\n";
      }
      if ( m_state != Playing && re_playing.search (m_stdout_buffer) >= 0 )
      {
        if ( m_mute )
        {
          m_mute = false;
          mute();
        }
        else
          setVolume (m_volume);
      }
      if ( m_state != Playing && re_start.search (m_stdout_buffer) >= 0 )
        setState (Playing);
//    kdDebug() << "matching a_or_v regex\n";
      if ( re_a_or_v.search (m_stdout_buffer) >= 0 )
      {
        if ( m_state != Playing && ! m_paused )
          setState (Playing);
	if ( m_quit && ! sent )
	{
          m_player -> writeStdin ("quit\n", 5);
	  sent = true;
	}
//      kdDebug() << "regex matched\n";
        if ( re_a_and_v.search (m_stdout_buffer) >= 0 )
        {
          ftime = stringToFloat (re_a_and_v.cap (1));
          float ftime2 = stringToFloat (re_a_and_v.cap (2));
          if ( ftime2 > ftime )
            ftime = ftime2;
        }
        else
        {
          ftime = stringToFloat (re_a_or_v.cap (1));
//        kdDebug() << "match: " << re_a_or_v.cap (1) << "ftime: " << ftime << "\n";
        }
//      if ( ftime > m_length && ftime >= 10 && m_length > 0 )
        if ( ftime > m_length && m_length >= 10 )
          m_length = ftime;
        if ( ftime != m_position )
        {
          m_position = ftime;
          emit progressChanged (m_position);
//        kdDebug() << "position: " << m_position << " length: " << m_length << "\n";
        }
      }
//    kdDebug() << "end of loop\n";
      m_stdout_line_length = 0;
      len -= cr - str + 1;
      str = cr + 1;
    }
    while ( cr < lf );
  }
//kdDebug() << "normal return\n";
}

void KPlayerProcess::onReceivedHelperStdout (KProcess* proc, char* str, int len)
{
  if ( proc != m_helper )
  {
    char buf [1025];
    if ( len > 1024 )
      len = 1024;
    memcpy (buf, str, len);
    buf [len] = 0;
    if ( re_exiting.search (buf) < 0 )
      proc -> writeStdin ("quit\n", 5);
    return;
  }
  bool sent = false;
  float ftime;
//kdDebug() << "helper stdout received length: " << len << "\n";
//kdDebug() << m_helper_line_length << "/" << m_helper_buffer_length << ": " << m_helper_stdout_buffer << "\n";
  while ( len > 0 )
  {
    char* lf = (char*) memchr (str, '\n', len);
    if ( ! lf )
      lf = str + len;
    char* cr = str;
    do
    {
      cr = (char*) memchr (str, '\r', lf - str);
      if ( ! cr )
        cr = lf;
      if ( cr - str < 2 && ! *str && ! m_helper_line_length )
        continue;
      if ( cr - str + m_helper_line_length >= m_helper_buffer_length )
      {
        char* old_buffer = m_helper_stdout_buffer;
        m_helper_buffer_length = cr - str + m_helper_line_length + 1;
        kdDebug() << "new helper buffer length: " << m_helper_buffer_length << "\n";
        m_helper_stdout_buffer = new char [m_helper_buffer_length];
        if ( m_helper_line_length )
          memcpy (m_helper_stdout_buffer, old_buffer, m_helper_line_length);
        delete [] old_buffer;
      }
      if ( cr > str )
      {
        memcpy (m_helper_stdout_buffer + m_helper_line_length, str, cr - str);
        m_helper_line_length += cr - str;
      }
      if ( cr - str == len )
      {
//      kdDebug() << "stdout return\n";
        return;
      }
      m_helper_stdout_buffer [m_helper_line_length] = 0;
      if ( *m_helper_stdout_buffer ) // && cr == lf
      {
        write (STDOUT_FILENO, m_helper_stdout_buffer, m_helper_line_length);
        write (STDOUT_FILENO, "\n", 1);
      }
//    kdDebug() << "Buffer: " << m_helper_stdout_buffer << "\n";
//    kdDebug() << "matching video regex\n";
      if ( re_video.search (m_helper_stdout_buffer) >= 0 )
      {
        m_width = re_video.cap(1).toInt();
        m_height = re_video.cap(2).toInt();
        kdDebug() << "Width: " << m_width << " Height: " << m_height << "\n";
      }
      if ( re_ans_length.search (m_helper_stdout_buffer) >= 0 )
      {
        kdDebug() << "Matched ans_length: " << re_ans_length.cap (1) << " in " << re_ans_length.cap (0) << "\n";
        m_ans_length = re_ans_length.cap (1).toInt();
      }
//    kdDebug() << "matching a_or_v regex\n";
      if ( re_a_or_v.search (m_helper_stdout_buffer) >= 0 )
      {
//      kdDebug() << "regex matched\n";
        if ( re_a_and_v.search (m_helper_stdout_buffer) >= 0 )
        {
          ftime = stringToFloat (re_a_and_v.cap (1));
          float ftime2 = stringToFloat (re_a_and_v.cap (2));
          if ( ftime2 > ftime )
            ftime = ftime2;
        }
        else
        {
          ftime = stringToFloat (re_a_or_v.cap (1));
//        kdDebug() << "match: " << re_a_or_v.cap (1) << "ftime: " << ftime << "\n";
        }
        if ( ftime > m_length && ftime >= 10 && m_length > 0 )
          m_length = ftime;
        if ( ftime != m_helper_position )
        {
          m_helper_position = ftime;
          kdDebug() << "helper position: " << m_helper_position << " length: " << m_length << "\n";
        }
        if ( m_seek > 0 && ! sent )
        {
          if ( ++ m_seek_count < 5 )
            sent = true;
          else
            m_seek_count = 0;
          kdDebug() << "m_seek_count: " << m_seek_count << "\n";
        }
        if ( m_seek > 0 && m_seek < 500 && ftime >= 2 )
        {
          float estlength = ftime * 100 / m_seek;
          if ( m_length < estlength )
            m_length = estlength;
          kdDebug() << "estimated length: " << m_length << "\n";
        }
        if ( m_seek == 0 && ! sent )
        {
          m_helper -> writeStdin ("seek 99 1\n", 10);
//        m_helper -> writeStdin ("get_time_length\n", 16);
          kdDebug() << "seek 99 1\n";
          m_seek = 99;
          m_seek_count = 0;
          sent = true;
        }
        if ( m_seek == 99 && ! sent && m_length < 2 )
        {
          m_helper -> writeStdin ("seek 95 1\n", 10);
//        m_helper -> writeStdin ("get_time_length\n", 16);
          kdDebug() << "seek 95 1\n";
          m_seek = 95;
          m_seek_count = 0;
          sent = true;
        }
        if ( m_seek == 95 && ! sent && m_length < 2 )
        {
          m_helper -> writeStdin ("seek 90 1\n", 10);
//        m_helper -> writeStdin ("get_time_length\n", 16);
          kdDebug() << "seek 90 1\n";
          m_seek = 90;
          m_seek_count = 0;
          sent = true;
        }
        if ( m_seek == 90 && ! sent && m_length < 2 )
        {
          m_helper -> writeStdin ("seek 50 1\n", 10);
//        m_helper -> writeStdin ("get_time_length\n", 16);
          kdDebug() << "seek 50 1\n";
          m_seek = 50;
          m_seek_count = 0;
          sent = true;
        }
        if ( m_seek < 100 && m_length >= 2 )
        {
          m_helper -> writeStdin ("seek 100 1\n", 11);
//        m_helper -> writeStdin ("get_time_length\n", 16);
          kdDebug() << "seek 100 1\n";
          m_seek = 100;
          m_seek_count = 0;
          sent = true;
        }
        if ( (m_seek == 50 || m_seek == 100) && ! sent )
        {
          m_helper -> writeStdin ("quit\n", 5);
          kdDebug() << "quit\n";
          m_seek = 500;
          sent = true;
        }
      }
//    kdDebug() << "end of loop\n";
      m_helper_line_length = 0;
      len -= cr - str + 1;
      str = cr + 1;
    }
    while ( cr < lf );
  }
//kdDebug() << "normal return\n";
}

void KPlayerProcess::onReceivedStderr (KProcess*, char* str, int len)
{
  write (STDERR_FILENO, str, len);
//hide();
//show();
}
