/***************************************************************************
                          kplayerwidget.cpp
                          -----------------
    begin                : Sun Dec 01 2002
    copyright            : (C) 2002-2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <kfiledialog.h>
#include <kurlrequesterdlg.h>
#include <qapplication.h>

#include <kdebug.h>

#include "kplayerwidget.h"

extern void KPlayerSetX11EventFilter (void);
extern void KPlayerResetX11EventFilter (void);

static KPlayerWidget* CurrentKPlayerWidget = 0;

KPlayerWidget::KPlayerWidget (QWidget *parent, const char *name)
  : QWidget (parent, name), m_aspect (0, 0), m_size (0, 0)
{
  m_original_aspect = false;
  m_resizing = false;
  m_process = new KPlayerProcess;
  m_process -> setWinId (winId());
  connect (m_process, SIGNAL (stateChanged(KPlayerProcess::State)), this, SLOT (playerStateChanged(KPlayerProcess::State)));
//setFocusPolicy (QWidget::NoFocus);
//setEnabled (false);
  setEraseColor (QColor (0, 0, 0));
  CurrentKPlayerWidget = this;
  KPlayerSetX11EventFilter();
}

KPlayerWidget::~KPlayerWidget()
{
  if ( m_process )
    delete m_process;
  KPlayerResetX11EventFilter();
  if ( CurrentKPlayerWidget == this )
    CurrentKPlayerWidget = 0;
}

void KPlayerWidget::maintainOriginalAspect (bool original)
{
  setFixedAspect (0, 0);
  m_original_aspect = original; // && ! kPlayerProcess() -> size(). isEmpty();
  setVideoSize (size());
}

void KPlayerWidget::maintainCurrentAspect (bool current)
{
  if ( current && ! kPlayerProcess() -> size(). isEmpty() )
  {
    setFixedAspect (width(), height());
    setVideoSize (size());
  }
  else
    setFixedAspect (0, 0);
}

bool KPlayerWidget::getFixedAspect (int& horz, int& vert)
{
  horz = m_original_aspect ? kPlayerProcess() -> width() : m_aspect.width();
  vert = m_original_aspect ? kPlayerProcess() -> height() : m_aspect.height();
  return isFixedAspect();
}

void KPlayerWidget::setFixedAspect (int horz, int vert)
{
  if ( kPlayerProcess() -> size(). isEmpty() )
    horz = vert = 0;
  m_aspect.setWidth (horz);
  m_aspect.setHeight (vert);
  m_original_aspect = false;
  setVideoSize (size());
}

void KPlayerWidget::setVideoSize (const QSize& size, bool canGrow, bool horizontally)
{
  m_size = size;
  adjustAspect (m_size, canGrow, horizontally);
  kdDebug() << "Video size " << size.width() << "x" << size.height()
    << ", adjusted size " << m_size.width() << "x" << m_size.height() << "\n";
}

void KPlayerWidget::adjustWidth (QSize& size) const
{
  if ( m_original_aspect )
    size.setWidth (size.height() * kPlayerProcess() -> width() / kPlayerProcess() -> height());
  else
    size.setWidth (size.height() * m_aspect.width() / m_aspect.height());
}

void KPlayerWidget::adjustHeight (QSize& size) const
{
  if ( m_original_aspect )
    size.setHeight (size.width() * kPlayerProcess() -> height() / kPlayerProcess() -> width());
  else
    size.setHeight (size.width() * m_aspect.height() / m_aspect.width());
}

void KPlayerWidget::adjustAspect (QSize& size, bool canGrow, bool horizontally) const
{
  if ( ! isFixedAspect() || size.isEmpty() )
    return;
  if ( ! canGrow )
    horizontally = size.width() * (m_original_aspect ? kPlayerProcess() -> height() : m_aspect.height())
      > size.height() * (m_original_aspect ? kPlayerProcess() -> width() : m_aspect.width());
  if ( horizontally )
    adjustWidth (size);
  else
    adjustHeight (size);
}

/*void KPlayerWidget::resize (int w, int h)
{
  kdDebug() << "resize: Minimum Size: " << minimumWidth() << "x" << minimumHeight()
    << ", Maximum Size: " << maximumWidth() << "x" << maximumHeight()
    << ", Size: " << width() << "x" << height() << "\n";
  QWidget::resize (w, h);
  kdDebug() << "resize: Minimum Size: " << minimumWidth() << "x" << minimumHeight()
    << ", Maximum Size: " << maximumWidth() << "x" << maximumHeight()
    << ", Size: " << width() << "x" << height() << "\n";
}

void KPlayerWidget::setGeometry (int x, int y, int w, int h)
{
  if ( x == this -> x() && y == this -> y() && w == width() && h == height() )
  {
    QWidget::setGeometry (x, y, w, h);
    return;
  }
  kdDebug() << "setgeo: Minimum Size: " << minimumWidth() << "x" << minimumHeight()
    << ", Maximum Size: " << maximumWidth() << "x" << maximumHeight()
    << ", Size: " << width() << "x" << height() << "\n";
  QWidget::setGeometry (x, y, w, h);
  kdDebug() << "setgeo: Minimum Size: " << minimumWidth() << "x" << minimumHeight()
    << ", Maximum Size: " << maximumWidth() << "x" << maximumHeight()
    << ", Size: " << width() << "x" << height() << "\n";
}*/

bool KPlayerWidget::openFile (void)
{
  QString name = KFileDialog::getOpenFileName (QString::null,
    "*|All files\n*.avi|AVI files\n*.mpg *.mpeg|MPEG files\n*.ogg|OGG files\n*.mp3|MP3 files");
  if ( name.isEmpty() )
    return false;
  m_process -> stop();
  m_process -> setUrl (name);
  return true;
}

bool KPlayerWidget::openUrl (void)
{
  QString name = KURLRequesterDlg::getURL().url();
  if ( name.isEmpty() )
    return false;
  m_process -> stop();
  m_process -> setUrl (name);
  return true;
}

bool KPlayerWidget::openSubtitle (void)
{
  QString name = KFileDialog::getOpenFileName (QString::null, "*|All files\n*.smi|SMI files\n*.sub|SUB files");
  if ( name.isEmpty() )
    return false;
  m_process -> stop();
  m_process -> setSubtitleUrl (name);
  return true;
}

bool KPlayerWidget::openSubtitleUrl (void)
{
  QString name = KURLRequesterDlg::getURL().url();
  if ( name.isEmpty() )
    return false;
  m_process -> stop();
  m_process -> setSubtitleUrl (name);
  return true;
}

// WM_NORMAL_HINTS, XSizeHints, XSetWMNormalHints() ???

void KPlayerWidget::zoomIn (void)
{
  setVideoSize (size() + kPlayerProcess() -> size() / 2);
}

void KPlayerWidget::zoomOut (void)
{
  setVideoSize (size() - kPlayerProcess() -> size() / 2);
}

bool KPlayerWidget::isZoomFactor (int m, int d) const
{
  QSize size = kPlayerProcess() -> size() * m / d;
  adjustAspect (size, true);
  return size == m_size;
}

void KPlayerWidget::setZoomFactor (int m, int d)
{
  setVideoSize (kPlayerProcess() -> size() * m / d);
}

/*bool KPlayerWidget::event (QEvent* ev)
{
//kdDebug() << "KPlayerWidget event: type " << ev -> type()
//  << " spontaneous " << ev -> spontaneous() << "\n";
  return QWidget::event (ev);
}*/

void KPlayerWidget::closeEvent (QCloseEvent* event)
{
  m_process -> stop();
  QWidget::closeEvent (event);
}

void KPlayerWidgetResizeHandler (bool resizing)
{
  if ( CurrentKPlayerWidget )
    CurrentKPlayerWidget -> resizeHandler (resizing);
}

void KPlayerWidget::resizeHandler (bool resizing)
{
  m_resizing = resizing;
  if ( ! resizing && isFixedAspect() )
    QApplication::postEvent (this, new QResizeEvent (size(), size()));
}

void KPlayerWidget::resizeEvent (QResizeEvent* event)
{
  kdDebug() << "KPlayerWidget::resizeEvent\n";
  if ( ! kPlayerProcess() -> size().isEmpty() )
  {
    hide();
    show();
  }
  QWidget::resizeEvent (event);
  kdDebug() << "revent: new size " << event -> size(). width() << "x" << event -> size(). height()
    << ", old size " << event -> oldSize(). width() << "x" << event -> oldSize(). height() << "\n";
  if ( ! m_resizing )
    emit resized();
}

void KPlayerWidget::playerStateChanged (KPlayerProcess::State state)
{
  kdDebug() << "State change received by the widget: " << state << "\n";
  if ( state == KPlayerProcess::Playing && ! kPlayerProcess() -> size().isEmpty() )
  {
    // hackery needed to ensure proper display
    hide();
    QSize size (size());
    resize (100, 100);
    resize (size);
    show();
    kdDebug() << "Widget size: " << width() << "x" << height() << "\n";
  }
}

KPlayerWorkspace::KPlayerWorkspace (QWidget* parent, const char* name)
  : QWidget (parent, name)
{
  m_widget = new KPlayerWidget (this);
  m_free_resize_mode = true;
  setEraseColor (QColor (0, 0, 0));
//setFocusPolicy (QWidget::NoFocus);
//setEnabled (false);
}

void KPlayerWorkspace::setVideoSize (void)
{
  m_widget -> setVideoSize (size(), false);
  m_widget -> setGeometry ((width() - m_widget -> videoSize().width()) / 2,
    (height() - m_widget -> videoSize().height()) / 2,
    m_widget -> videoSize().width(), m_widget -> videoSize().height());
}

void KPlayerWorkspace::resizeEvent (QResizeEvent* event)
{
  QWidget::resizeEvent (event);
  if ( m_free_resize_mode )
    m_widget -> setGeometry (0, 0, width(), height());
  else
    setVideoSize();
}
